@@ -29,112 +29,77 @@ ROADMAP:
  - main should also parse command line arguments
 
  ### Version Control integration
- ! automatize rerunning an experiment by checking out the appropriate version and feed the parameters
+ ! automatize rerunning an experiment by checking out the appropriate version
+   and feed the parameters
  ? gather versions of dependencies
 
  ### Display results
  V should be decoupled from console/pc we are running on
- V figure out how to incorporate plots
- V Make very long stages deliver a stream of data to inspect their behaviour live
  ? maybe start a webserver to watch results
- ? maybe incorporate self-updating plots into ipython-notebook
-
 """
 
 from __future__ import division, print_function, unicode_literals
 
-
-from configobj import ConfigObj
 from copy import copy
+import inspect
+import os
 import log
-import logging
 from matplotlib import pyplot as plt
 import numpy as np
-from StringIO import StringIO
+import time
 
-from caches import CacheStub
-from mlizard.stage import StageFunctionOptionsView, StageFunction
+from report import Report, PlainTextReportFormatter
+from stage import StageFunctionOptionsView, StageFunction
 
-__all__ = ['Experiment', 'createExperiment']
+__all__ = ['Experiment']
 
 RANDOM_SEED_RANGE = 0, 1000000
 
-
-
-
-def createExperiment(name = "Experiment", config_file=None, config_string=None, logger=None, seed=None, cache=None):
-    # setup logging
-    if logger is None:
-        logger = logging.getLogger(name)
-        logger.setLevel(logging.INFO)
-        ch = logging.StreamHandler()
-        ch.setLevel(logging.INFO)
-        formatter = logging.Formatter('%(levelname)s - %(name)s - %(message)s')
-        ch.setFormatter(formatter)
-        logger.addHandler(ch)
-        logger.info("No Logger configured: Using generic stdout Logger")
-
-    # reading configuration
-    options = ConfigObj(unrepr=True)
-    if config_file is not None:
-        if isinstance(config_file, basestring) :
-            logger.info("Loading config file {}".format(config_file))
-            options = ConfigObj(config_file, unrepr=True, encoding="UTF-8")
-        elif hasattr(config_file, 'read'):
-            logger.info("Reading configuration from file.")
-            options = ConfigObj(config_file, unrepr=True, encoding="UTF-8")
-    elif config_string is not None:
-        logger.info("Reading configuration from string.")
-        options = ConfigObj(StringIO(str(config_string)), unrepr=True, encoding="UTF8")
-
-    # get seed for random numbers in experiment
-    if seed is None:
-        if 'seed' in options:
-            seed = options['seed']
-        else:
-            seed = np.random.randint(*RANDOM_SEED_RANGE)
-            logger.warn("No Seed given. Using seed={}. Set in config "
-                        "file to repeat experiment".format(seed))
-
-    cache = cache or CacheStub()
-    results_logger = logging.getLogger("Results")
-    prng = np.random.RandomState(seed)
-    return Experiment(name, logger, results_logger, options, prng, cache)
-
-
-
-class OptionContext(object):
-    def __init__(self, options, stage_functions):
-        self.options = options
-        for sf in stage_functions:
-            sf_view = StageFunctionOptionsView(sf, options)
-            self.__setattr__(sf.func_name, sf_view)
-
-    def __enter__(self):
-        return self
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        pass
-
-
 class Experiment(object):
-    def __init__(self, name, message_logger, results_logger, options, prng, cache):
+    def __init__(self, name, message_logger, results_logger, options, cache,
+                 seed = None):
         self.name = name
         self.message_logger = message_logger
         self.results_logger = results_logger
         self.options = options
-        self.prng = prng
         self.cache = cache
         self.results_handler = log.ResultLogHandler()
         self.results_logger.addHandler(self.results_handler)
         self.stages = dict()
-        self.plots = []
+        self.main_stage = None
+        self.plot_functions = []
+        self.live_plots = []
+
+        if seed is None:
+            seed = np.random.randint(*RANDOM_SEED_RANGE)
+            message_logger.warn("No Seed given. Using seed={}. Set in config "
+                                "file to repeat experiment".format(seed))
+        self.seed = seed
+        self.prng = np.random.RandomState(seed)
 
     def optionset(self, section_name):
         options = copy(self.options)
         options.update(self.options[section_name])
         return OptionContext(options, self.stages.values())
 
+    def optionsets(self, section_names):
+        for sn in section_names:
+            o = self.optionset(sn)
+            o.__enter__()
+            yield o
+            o.__exit__(None, None, None)
+
+    def convert_to_stage_function(self, f):
+        if isinstance(f, StageFunction): # do nothing if it is already a stage
+            # do we need to allow beeing stage of multiple experiments?
+            return f
+        else :
+            stage_name = f.func_name
+            stage_msg_logger = self.message_logger.getChild(stage_name)
+            stage_results_logger = self.results_logger.getChild(stage_name)
+            stage_seed = self.prng.randint(*RANDOM_SEED_RANGE)
+            return StageFunction(stage_name, f, self.cache, self.options,
+                stage_msg_logger, stage_results_logger, stage_seed)
 
     def stage(self, f):
         """
@@ -153,30 +118,98 @@ class Experiment(object):
         - you pass an unexpected keyword argument
         - you provide multiple values for an argument
         - after all the filling an argument is still missing"""
-        if isinstance(f, StageFunction): # do nothing if it is already a stage
-            # do we need to allow beeing stage of multiple experiments?
-            return f
-        else :
-            stage_name = f.func_name
-            stage_msg_logger = self.message_logger.getChild(stage_name)
-            stage_results_logger = self.results_logger.getChild(stage_name)
-
-            stage_seed = self.prng.randint(*RANDOM_SEED_RANGE)
-            stage = StageFunction(stage_name, f, self.cache, self.options, stage_msg_logger, stage_results_logger, stage_seed)
-            self.stages[stage_name] = stage
-            return stage
+        stage = self.convert_to_stage_function(f)
+        self.stages[stage.__name__] = stage
+        return stage
 
     def plot(self, f):
         """decorator to generate plots"""
-        self.plots.append(f)
+        self.plot_functions.append(f)
         return f
 
+    def live_plot(self, f):
+        if not inspect.isgeneratorfunction(f):
+            raise TypeError("Live plots must be generator functions!")
+        self.live_plots.append(f)
+        self.results_handler.add_plot(f)
+
+    def set_paths(self, main_file):
+        if 'results_dir' in self.options:
+            self.results_dir = self.options.results_dir
+        else :
+            self.results_dir = os.path.dirname(main_file)
+        if not os.path.exists(self.results_dir):
+            self.message_logger.warn("results_dir '%s' does not exist. No results will be written.", self.results_dir)
+            self.results_dir = None
+
 
     def main(self, f):
+        assert self.main_stage is None, "Only one main stage is allowed!"
+        self.main_stage = self.convert_to_stage_function(f)
+        main_file = inspect.getabsfile(f)
+        self.set_paths(main_file)
         if f.__module__ == "__main__":
-            f()
-            for p in self.plots:
-                p(self.results_handler.results).show()
+            import sys
+            args = sys.argv[1:]
+            ######## run main #########
+            report = self(*args)
+            ###########################
+            self.write_report(report)
+            # show all plots and wait
             plt.ioff()
             plt.show()
-        return f
\ No newline at end of file
+            sys.exit(0)
+        return self
+
+    def __call__(self, *args, **kwargs):
+        report = Report(self.name)
+        self.message_logger.addHandler(report)
+        ######## call stage #########
+        result = self.main_stage.execute_function(args, kwargs, self.options)
+        #############################
+        self.message_logger.removeHandler(report)
+        report.seed = self.seed
+        report.options = self.options
+        report.main_result = result
+        report.logged_results = self.results_handler.results
+        report.stage_summary = [{'name' : s.__name__,
+                                 'execution_times' : s.execution_times}
+                                for s in self.stages.values()]
+        # plotting
+        plots = []
+        for p in self.plot_functions:
+            fig = p(self.results_handler.results)
+            fig.draw()
+            plots.append(fig)
+        report.plots = plots
+        report.end_time = time.time()
+        return report
+
+    def write_report(self, report):
+        formatter = PlainTextReportFormatter()
+        if 'report_filename' in self.options and self.results_dir:
+            report_path = os.path.join(self.results_dir, self.options['report_filename'])
+            with open(report_path, 'w') as f:
+                f.write(formatter.format(report))
+        else:
+            print(formatter.format(report))
+
+
+class OptionContext(object):
+    def __init__(self, options, stage_functions):
+        self.options = options
+        for sf in stage_functions:
+            sf_view = StageFunctionOptionsView(sf, options)
+            self.__setattr__(sf.func_name, sf_view)
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        pass
+
+    def keys(self):
+        return self.options.keys()
+
+    def __getitem__(self, item):
+        return self.options[item]
