@@ -7,27 +7,14 @@
 from __future__ import division, print_function, unicode_literals
 
 import logging
-import numpy as np
 from tempfile import NamedTemporaryFile
 
 from helpers import *
-
-from mlizard.caches import CacheStub
-from mlizard.experiment import Experiment, createExperiment
+from ..factory import createExperiment, create_basic_Experiment
 
 # don't gather logging spam
 logging.disable(logging.CRITICAL)
 
-NO_LOGGER = logging.getLogger('ignore')
-NO_LOGGER.disabled = 1
-
-def create_basic_Experiment(seed = 123456):
-    name = "TestExperiment"
-    options = {}
-    cache = CacheStub()
-    prng = np.random.RandomState(seed)
-    return Experiment(name, NO_LOGGER, NO_LOGGER, options, prng, cache)
-
 def test_Experiment_constructor_works():
     ex1 = create_basic_Experiment()
 
@@ -126,6 +113,18 @@ def test_stage_keeps_explicit_arguments():
 
     assert_equal(foo(0, beta=0), (0, 0))
 
+def test_stage_explicit_arguments_overwrite_all():
+    ex1 = create_basic_Experiment()
+    ex1.options["alpha"] = 0.7
+    ex1.options["beta"] = 1.2
+
+    @ex1.stage
+    def foo(alpha, beta=10):
+        return alpha, beta
+
+    assert_equal(foo(0, 0), (0, 0))
+
+
 @raises(TypeError)
 def test_stage_with_unexpected_kwarg_raises_TypeError():
     ex1 = create_basic_Experiment()
@@ -156,6 +155,27 @@ def test_stage_with_missing_arguments_raises_TypeError():
     #noinspection PyArgumentList
     foo(1)
 
+@raises(TypeError)
+def test_stage_with_vararg_raises_TypeError():
+    ex1 = create_basic_Experiment()
+    @ex1.stage
+    def foo(*args): pass
+
+@raises(TypeError)
+def test_stage_with_kw_wildcard_raises_TypeError():
+    ex1 = create_basic_Experiment()
+    @ex1.stage
+    def foo(**kwargs): pass
+
+@raises(TypeError)
+def test_stage_with_too_many_args_raises_TypeError():
+    ex1 = create_basic_Experiment()
+    @ex1.stage
+    def foo(a, b=5): pass
+
+    foo(1, 2, 3)
+
+
 def test_experiment_reads_options_from_file():
     with NamedTemporaryFile() as f:
         f.write("""
@@ -182,11 +202,11 @@ def test_experiment_reads_options_from_file():
 def test_experiment_reads_seed_from_file():
     with NamedTemporaryFile() as f:
         f.write("""
-        seed=12345
+        seed=54321
         """)
         f.flush()
         ex1 = createExperiment(config_file=f.name)
-        assert_equal(ex1.seed, 12345)
+        assert_equal(ex1.seed, 54321)
 
 
 def test_fill_args_seeds_deterministic():
